#include "../SDK/foobar2000.h"
#include "../helpers/helpers.h"
#include "../../pfc/pfc.h"
#include <commdlg.h>

DECLARE_COMPONENT_VERSION("Cuesheet Creator", "0.4.6",
	"Primary Author:\n"
	"  Brian Peyton\n"
	"Additional Coding:\n"
	"  Neil Popham\n"
	"  Haru Ayana@Re:harmonize\n"
	"Based on the Component development tutorial step 1 by:\n"
	"  Holger Stenger");

enum outputEncoding { utf8, ansi, album_utf8, album_ansi };

// {21C82BA8-1F53-4276-A8C2-8BB2A0694993}
static const GUID guid_contextmenu_item_simple_cuesheet_creator = 
{ 0x21c82ba8, 0x1f53, 0x4276, { 0xa8, 0xc2, 0x8b, 0xb2, 0xa0, 0x69, 0x49, 0x93 } };

bool getSaveFilename(char * fileName, int size, const char * title, outputEncoding &encoding) {

	const int MAXPATH = 1024;
    wchar_t pathname[MAXPATH];
    OPENFILENAME ofn;

    ZeroMemory(&ofn,sizeof(ofn));
	lstrcpy((wchar_t*)pathname, pfc::stringcvt::string_wide_from_utf8(title, MAXPATH).get_ptr());

    ofn.lStructSize = sizeof(ofn);
    ofn.lpstrFilter = L"UTF-8 Cuesheet (*.cue)\0*.cue\0ANSI Cuesheet (*.cue)\0*.cue\0UTF-8 Album Image Cuesheet (*.cue)\0*.cue\0ANSI Album Image Cuesheet (*.cue)\0*.cue\0\0";
    ofn.nFilterIndex = 1;
	ofn.lpstrDefExt = L"cue";
	ofn.lpstrFile = (wchar_t*)pathname;
    ofn.nMaxFile = MAXPATH;
    ofn.lpstrTitle = L"Save Cuesheet";
    ofn.Flags=OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_OVERWRITEPROMPT;

	if(GetSaveFileName(&ofn)){
		pfc::stringcvt::convert_wide_to_utf8(fileName, MAXPATH*2, pathname, sizeof(pathname));
		switch (ofn.nFilterIndex) {
		case 1:
			encoding = utf8;
			break;
		case 2:
			encoding = ansi;
			break;
		case 3:
			encoding = album_utf8;
			break;
		case 4:
			encoding = album_ansi;
			break;
		} // end switch
		return true;
	} else {
		return false;
    } // end if

}

class contextmenu_item_simple_cuesheet_creator : public contextmenu_item_simple
{
private:
	bool create_cuesheet(const pfc::list_base_const_t<metadb_handle_ptr> & p_data)
	{
		const unsigned char UTF8_BOM[3] = {0xEF, 0xBB, 0xBF};
		char fileName[1024];
		outputEncoding encoding = utf8;
		cue_creator::t_entry_list entry_list;
		pfc::string_formatter cuesheet;
		service_ptr_t<file> outfile;
		abort_callback_impl p_abort;
		double index_position = 0;
		file_info_impl tmp_info;

		p_data.get_item(0)->get_info(tmp_info);
		pfc::string8 title = tmp_info.meta_get("ALBUM", 0);
		title.add_string(".cue");
		title.fix_filename_chars();
		if(!getSaveFilename(fileName, sizeof(fileName), title.get_ptr(), encoding)) return false;
		pfc::string8 old_file;
		double old_length;
		for (int i = 0; i != p_data.get_count(); ++i) {
			cue_creator::t_entry entry;
			metadb_handle_ptr item = p_data.get_item(i);
			item->get_info(entry.m_infos);
			if (encoding == utf8 || encoding == ansi) {
				entry.m_file = pfc::string_filename_ext(item->get_path());
			} else {
				entry.m_file.set_string("CDImage.");
				entry.m_file.add_string(pfc::string_extension(item->get_path()));
			}
			if (stricmp_utf8(old_file, entry.m_file) == 0) {
				index_position += old_length;
			} else {
				pfc::string8 pregap_string = entry.m_infos.info_get("pregap");
				double pregap = cuesheet_parse_index_time_e(pregap_string, pregap_string.get_length());
				index_position = pregap;
			}
			entry.m_index_list.reset();
			entry.m_index_list.from_infos(entry.m_infos, index_position);
			entry.m_track_number = i+1;
			entry.m_flags = NULL;
			entry_list.insert_last(entry);
			old_file = entry.m_file;
			old_length = item->get_length();
		}
		cue_creator::create(cuesheet, entry_list);
		filesystem::g_open_write_new(outfile, (const char*)fileName, p_abort);
		if (encoding == utf8 || encoding == album_utf8) {
			// BOM
			outfile->write(UTF8_BOM, sizeof(UTF8_BOM), p_abort);
			outfile->write_string_raw(cuesheet.get_ptr(), p_abort);
		} else {
			const char* tmp = cuesheet.get_ptr();
			outfile->write_string_raw(pfc::stringcvt::string_ansi_from_utf8(tmp, cuesheet.length()).get_ptr(), p_abort);
		}
		return true;
	}

public:
	virtual t_enabled_state get_enabled_state(unsigned p_index)
	{
		return contextmenu_item::DEFAULT_ON;
	}

	virtual unsigned get_num_items()
	{
		return 1;
	}

	virtual void get_item_name(unsigned p_index,pfc::string_base & p_out)
	{
		p_out = "Save as cuesheet...";
	}

	virtual void get_item_default_path(unsigned p_index,pfc::string_base & p_out)
	{
		p_out = "Utils";
	}

	virtual void context_command(unsigned p_index,const pfc::list_base_const_t<metadb_handle_ptr> & p_data,const GUID& p_caller)
	{
		create_cuesheet(p_data);
	}

	virtual bool context_get_display(unsigned p_index,const pfc::list_base_const_t<metadb_handle_ptr> & p_data,pfc::string_base & p_out,unsigned & p_displayflags,const GUID & p_caller) {
		PFC_ASSERT(p_index>=0 && p_index<get_num_items());
		get_item_name(p_index,p_out);
		return true;
	}

	virtual GUID get_item_guid(unsigned p_index)
	{
		return guid_contextmenu_item_simple_cuesheet_creator;
	}

	virtual bool get_item_description(unsigned p_index,pfc::string_base & p_out)
	{
		p_out = "Creates cuesheet from select items.";
		return true;
	}
};

static contextmenu_item_factory_t<contextmenu_item_simple_cuesheet_creator> g_contextmenu_item_simple_cuesheet_creator_factory;